function clearMask(obj, height, width, z)
% function clearMask(obj)
% Clear the 'Mask' layer. It is also possible to specify
% the area where the 'Mask' layer should be cleared.
%
% Parameters:
% height: [@em optional] vector of heights, for example [1:imageData.height]
% width: [@em optional] vector of width, for example [1:imageData.width]
% z: [@em optional] vector of z-values, for example [1:imageData.no_stacks]
%
% Return values:
%

%| 
% @b Examples:
% @code imageData.clearMask();  // clear the mask layer @endcode
% @code clearMask(obj);   // Call within the class; clear the mask layer @endcode

% Copyright (C) 30.10.2013, Ilya Belevich (ilya.belevich @ helsinki.fi)
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 01.10.2014, added subarea

if strcmp(obj.model_type, 'uint6')  % 63 materials model type
    if isnan(obj.model(1)); return; end;    % selection is disabled
    if nargin <= 1  % clear whole Mask
        obj.model = bitset(obj.model, 7, 0);
    else % clear part of the Mask
        obj.model(height,width,z) = bitset(obj.model(height,width,z), 7, 0);
    end
else                                % 255 materials model type
    if nargin <= 1 % clear whole Mask
        obj.maskImg = NaN;
        %obj.maskImg = zeros(size(obj.img,1),size(obj.img,2),size(obj.img,4),'uint8');
    else    % clear part of the Mask
        obj.maskImg(height,width,z) = 0;
    end
end

if nargin <= 1  % extra things after clearing the whole Mask
        obj.maskExist = 0;
        [pathstr, ~] = fileparts(obj.img_info('Filename'));
        obj.maskImgFilename = fullfile(pathstr, 'newMask.mask');    
end

end