function result = deleteSlice(obj, sliceNumber)
% function deleteSlice(obj, sliceNumber)
% Delete specified slice from the dataset.
%
% Parameters:
% sliceNumber: the number of the slice to delete
%
% Return values:
% result: result of the function, @b 0 fail, @b 1 success

%| 
% @b Examples:
% @code imageData.deleteSlice(3);     // delete the 3rd slice from the dataset @endcode
% @code deleteSlice(obj, 3); // Call within the class; delete the 3rd slice from the dataset @endcode

% Copyright (C) 30.10.2013, Ilya Belevich (ilya.belevich @ helsinki.fi)
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.

% Updates
% 


result = 0;
maxSliceNumber = size(obj.img, obj.orientation);
if sliceNumber > maxSliceNumber
    msgbox(sprintf('The maximal slice number is %d!', maxSliceNumber),'Wrong slice number','error','modal');
    return;
end

h = waitbar(0,sprintf('Deleting slice number(s) %s\nPlease wait...', num2str(sliceNumber)),'Name','Deleting the slice...');
maxZ = size(obj.img,4);
maxH = size(obj.img,1);
maxW = size(obj.img,2);
% delete slice from obj.img
if obj.orientation == 4     % xy orientation
    obj.img=obj.img(:,:,:,setdiff(1:maxZ,sliceNumber));
elseif obj.orientation == 1     % zx orientation
    obj.img=obj.img(setdiff(1:maxH,sliceNumber),:,:,:);
elseif obj.orientation == 2     % zy orientation
    obj.img=obj.img(:,setdiff(1:maxW,sliceNumber),:,:);
end
waitbar(0.3, h);

% delete slice from selection
if ~isnan(obj.selection(1))
    if obj.orientation == 4     % xy orientation
        obj.selection = obj.selection(:,:,setdiff(1:maxZ,sliceNumber));
    elseif obj.orientation == 1     % zx orientation
        obj.selection = obj.selection(setdiff(1:maxH,sliceNumber),:,:);
    elseif obj.orientation == 2     % zy orientation
        obj.selection = obj.selection(:,setdiff(1:maxW,sliceNumber),:);
    end
end
waitbar(0.5, h);

% delete slice from model
if ~isnan(obj.model(1))
    if obj.orientation == 4     % xy orientation
        obj.model = obj.model(:,:,setdiff(1:maxZ,sliceNumber));
    elseif obj.orientation == 1     % zx orientation
        obj.model = obj.model(setdiff(1:maxH,sliceNumber),:,:);
    elseif obj.orientation == 2     % zy orientation
        obj.model = obj.model(:,setdiff(1:maxW,sliceNumber),:);
    end
end
waitbar(0.7, h);

% delete slice from mask
if ~isnan(obj.maskImg(1))
    if obj.orientation == 4     % xy orientation
        obj.maskImg = obj.maskImg(:,:,setdiff(1:maxZ,sliceNumber));
    elseif obj.orientation == 1     % zx orientation
        obj.maskImg = obj.maskImg(setdiff(1:maxH,sliceNumber),:,:);
    elseif obj.orientation == 2     % zy orientation
        obj.maskImg = obj.maskImg(:,setdiff(1:maxW,sliceNumber),:);
    end
end
waitbar(0.9, h);
% update obj.height, obj.width, etc
obj.height = size(obj.img, 1);
obj.width = size(obj.img, 2);
obj.no_stacks = size(obj.img, 4);
obj.img_info('Height') = size(obj.img, 1);
obj.img_info('Width') = size(obj.img, 2);
obj.img_info('Stacks') = size(obj.img, 4);

% update I.slices
currSlices = obj.slices;
obj.slices = [1, obj.height; 1, obj.width; 1, size(obj.img,3); 1,size(obj.no_stacks,4)];

if currSlices(obj.orientation) > size(obj.img, obj.orientation)
    obj.slices(obj.orientation,:) = size(obj.img, obj.orientation);
else
    obj.slices(obj.orientation,:) = currSlices(obj.orientation,:);
end

obj.current_yxz(1) = min([obj.current_yxz(1) obj.height]);
obj.current_yxz(2) = min([obj.current_yxz(2) obj.width]);
obj.current_yxz(3) = min([obj.current_yxz(3) obj.no_stacks]);

% update bounding box
obj.updateBoundingBox();

% update SliceName key in the img_info 
if isKey(obj.img_info, 'SliceName')
    sliceNames = obj.img_info('SliceName');
    if numel(obj.img_info('SliceName')) > 1
        sliceNames(sliceNumber) = [];
        obj.img_info('SliceName') = sliceNames;
    end
end

waitbar(1, h);
delete(h);
result = 1;
end