function result = setDataset(obj, type, dataset, orient, col_channel, options)
% function result = setDataset(obj, type, dataset, orient, col_channel, options)
% Update complete 4D dataset, @b LEGACY @b function, please use
% imageData.setData4D instead!
%
% Parameters:
% type: type of the dataset to update, 'image', 'model', 'mask', 'selection', 'everything' ('model','mask' and 'selection' for imageData.model_type==''uint6'' only)
% dataset: a 4D stack with images
% orient: [@em optional], can be @em NaN
% @li when @b 0 (@b default) returns the dataset transposed to the current orientation (obj.orientation)
% @li when @b 1 returns transposed dataset to the zx configuration: [y,x,c,z] -> [x,z,c,y]
% @li when @b 2 returns transposed dataset to the zy configuration: [y,x,c,z] -> [y,z,c,y]
% @li when @b 3 not used
% @li when @b 4 returns original dataset to the yx configuration: [y,x,c,z]
% @li when @b 5 not used% col_channel: [@em optional], can be @e NaN;
% @li when @b type is 'image', col_channel is a vector with color numbers to set, when @b NaN [@e default] update set the
% colors selected in the imageData.slicesColor variable; when @em 0 - update all colors of the dataset.
% @li when @b type is 'model' col_channel may be @em NaN - to update all materials of the model or an integer to update specific material.
% options: [@em optional], a structure with extra parameters
% @li .blockModeSwitch -> override the imageData.blockModeSwitch (@b 0 - update full dataset, @b 1 - update only the shown area)
% @li .y -> [@em optional], [ymin, ymax] of the part of the dataset to set;
% @li .x -> [@em optional], [xmin, xmax] of the part of the dataset to set
% @li .z -> [@em optional], [zmin, zmax] of the part of the dataset to set
%
% Return values:
% result: -> @b 1 - success, @b 0 - error

%|
% @b Examples:
% @code result = handles.Img{handles.Id}.I.setDataset('image', dataset);      // set the 4D dataset into 5D at the current time point  @endcode
% @code result = handles.Img{handles.Id}.I.setDataset('image', dataset, 0, NaN, options.blockModeSwitch=1); // set the croped to the viewing window part of the dataset, with shown colors @endcode
% @code result = handles.Img{handles.Id}.I.setDataset('image', dataset, 1, 2); // set complete transposed (XZ) dataset with only second color channel @endcode
% @code result = obj.setDataset('image', dataset);      // Call within the class; set the complete dataset  @endcode
% @attention @b sensitive to the @code imageData.blockModeSwitch @endcode
% @attention @b not @b sensitive to the shown ROI
% @attention for normal calls it is recommended to use ib_setDataset.m wrapper function, as @code
% ib_setDataset('image', dataset, handles); @endcode

% Copyright (C) 20.06.2014, Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 20.08.2015, Ilya Belevich, Simplified syntax
% 03.09.2015, Ilya Belevich updated to use imageData.setData4D method

result = 0; %#ok<NASGU>
if nargin < 6; options.blockModeSwitch = obj.blockModeSwitch; end;
if nargin < 5; col_channel = NaN; end;
if nargin < 4; orient=NaN; end;

time = 1;

if orient==0 || isnan(orient); orient=obj.orientation; end;

if ~isfield(options, 'blockModeSwitch'); options.blockModeSwitch = obj.blockModeSwitch; end;
if isfield(options, 'y'); options.blockModeSwitch = 1; end;

if strcmp(type,'image')
    if isnan(col_channel); col_channel=obj.slicesColor; end;
    if col_channel(1) == 0;  col_channel = 1:size(obj.img,3); end;
end

options.t = [time time];   % define the time point
result = obj.setData4D(type, dataset, time, orient, col_channel, options);

% if ~strcmp(obj.model_type, 'uint6')  % uint8/int8 type of the model
%     if options.blockModeSwitch == 0     % set the full size dataset
%         if strcmp(type,'image')
%             if replaceDatasetSwitch
%                 if permuteSw == 0 || obj.orientation==4 % yx
%                     obj.img = dataset;
%                 else    % get permuted dataset
%                     if obj.orientation==1     % xz
%                         obj.img = permute(dataset ,[4 1 3 2]);
%                     elseif obj.orientation==2 % yz
%                         obj.img = permute(dataset,[1 4 3 2]);
%                     end
%                 end
%             else
%                 if permuteSw == 0 || obj.orientation==4 % yx
%                     obj.img(:,:,col_channel,:) = dataset;
%                 else    % get permuted dataset
%                     if obj.orientation==1     % xz
%                         obj.img(:,:,col_channel,:) = permute(dataset ,[4 1 3 2]);
%                     elseif obj.orientation==2 % yz
%                         obj.img(:,:,col_channel,:) = permute(dataset,[1 4 3 2]);
%                     end
%                 end
%             end
%         elseif strcmp(type,'model')
%             if permuteSw == 0 || obj.orientation==4 % yx
%                 if ~isnan(col_channel)     % take only specific object
%                     obj.model(obj.model == col_channel) = 0;
%                     obj.model(dataset == 1) = col_channel;
%                 else
%                     obj.model = dataset;
%                 end
%             else    % get permuted dataset
%                 if obj.orientation==1     % xz
%                     if ~isnan(col_channel)     % take only specific object
%                         dataset = permute(dataset ,[3 1 2]);
%                         obj.model(obj.model == col_channel) = 0;
%                         obj.model(dataset == 1) = col_channel;
%                     else
%                         obj.model = permute(dataset ,[3 1 2]);
%                     end
%                 elseif obj.orientation==2 % yz
%                     if ~isnan(col_channel)     % take only specific object
%                         dataset = permute(dataset,[1 3 2]);
%                         obj.model(obj.model == col_channel) = 0;
%                         obj.model(dataset == 1) = col_channel;
%                     else
%                         obj.model = permute(dataset,[1 3 2]);
%                     end
%                 end
%             end
%         elseif strcmp(type,'mask')
%             if permuteSw == 0 || obj.orientation==4 % yx
%                 obj.maskImg = dataset;
%             else    % get permuted dataset
%                 if obj.orientation==1     % xz
%                     obj.maskImg = permute(dataset ,[3 1 2]);
%                 elseif obj.orientation==2 % yz
%                     obj.maskImg = permute(dataset,[1 3 2]);
%                 end
%             end
%             obj.maskExist = 1;
%         elseif strcmp(type,'selection')
%             if permuteSw == 0 || obj.orientation==4 % yx
%                 obj.selection = dataset;
%             else    % get permuted dataset
%                 if obj.orientation==1     % xz
%                     obj.selection = permute(dataset ,[3 1 2]);
%                 elseif obj.orientation==2 % yz
%                     obj.selection = permute(dataset,[1 3 2]);
%                 end
%             end
%         end
%     else        % set the shown block
%         if isfield(options, 'y');
%             getDatDimOpt.blockModeSwitch = 0;
%             [yVal,xVal,~,zVal] = obj.getDatasetDimensions('image', NaN, NaN, getDatDimOpt);
%             Xlim = [max([options.x(1) 1]) min([options.x(2) xVal])];
%             Ylim = [max([options.y(1) 1]) min([options.y(2) yVal])];
%             Zlim = [max([options.z(1) 1]) min([options.z(2) zVal])];
%         else
%             if obj.orientation==1     % xz
%                 Xlim = ceil(obj.axesY);
%                 Ylim = [1 size(obj.img, obj.orientation)];
%                 Zlim = ceil(obj.axesX);
%             elseif obj.orientation==2 % yz
%                 Xlim = [1 size(obj.img, obj.orientation)];
%                 Ylim = ceil(obj.axesY);
%                 Zlim = ceil(obj.axesX);
%             elseif obj.orientation==4 % yx
%                 Xlim = ceil(obj.axesX);
%                 Ylim = ceil(obj.axesY);
%                 Zlim = [1 size(obj.img, obj.orientation)];
%             end
%             Xlim = [max([Xlim(1) 1]) min([Xlim(2) obj.width])];
%             Ylim = [max([Ylim(1) 1]) min([Ylim(2) obj.height])];
%             Zlim = [max([Zlim(1) 1]) min([Zlim(2) obj.no_stacks])];
%         end
%         if strcmp(type,'image')
%             if permuteSw == 1 % permute
%                 if obj.orientation==1     % xz
%                     dataset = permute(dataset,[4 1 3 2]);
%                 elseif obj.orientation==2 % yz
%                     dataset = permute(dataset,[1 4 3 2]);
%                 end
%             end
%             obj.img(Ylim(1):Ylim(2),Xlim(1):Xlim(2),col_channel,Zlim(1):Zlim(2)) = dataset;
%         elseif strcmp(type,'model')
%             if permuteSw == 1   % permute dataset
%                 if obj.orientation==1     % xz
%                     dataset = permute(dataset,[3 1 2]);
%                 elseif obj.orientation==2 % yz
%                     dataset = permute(dataset,[1 3 2]);
%                 end
%             end
%             if ~isnan(col_channel)     % take only specific object
%                 currentDataset = obj.model(Zlim(1):Zlim(2),max([Ylim(1) 1]):min([Ylim(2) obj.width]),max([Xlim(1) 1]):min([Xlim(2) obj.no_stacks]));
%                 currentDataset(currentDataset == col_channel) = 0;
%                 currentDataset(dataset == 1) = col_channel;
%                 dataset = currentDataset;
%             end
%             obj.model(Ylim(1):Ylim(2),Xlim(1):Xlim(2),Zlim(1):Zlim(2)) = dataset;
%         elseif strcmp(type,'mask')
%             if obj.maskExist == 0  % create maskImg
%                 obj.maskImg = zeros(size(obj.img,1),size(obj.img,2),size(obj.img,4),'uint8');
%             end
%             if permuteSw == 1   % permute dataset
%                 if obj.orientation==1     % xz
%                     dataset = permute(dataset,[3 1 2]);
%                 elseif obj.orientation==2 % yz
%                     dataset = permute(dataset,[1 3 2]);
%                 end
%             end
%             obj.maskImg(Ylim(1):Ylim(2),Xlim(1):Xlim(2),Zlim(1):Zlim(2)) = dataset;
%             obj.maskExist = 1;
%         elseif strcmp(type,'selection')
%             if permuteSw == 1   % permute dataset
%                 if obj.orientation==1     % xz
%                     dataset = permute(dataset,[3 1 2]);
%                 elseif obj.orientation==2 % yz
%                     dataset = permute(dataset,[1 3 2]);
%                 end
%             end
%             obj.selection(Ylim(1):Ylim(2),Xlim(1):Xlim(2),Zlim(1):Zlim(2)) = dataset;
%         end
%     end
% else            % ************ uint6 model type
%     if islogical(dataset(1)); dataset = uint8(dataset); end;
%     if options.blockModeSwitch == 0     % set the full size dataset
%         if strcmp(type,'image')
%             if replaceDatasetSwitch
%                 if permuteSw == 0 && obj.orientation==4 % yx
%                     obj.img = dataset;
%                 else    % get permuted dataset
%                     if obj.orientation==1     % xz
%                         obj.img = permute(dataset ,[4 1 3 2]);
%                     elseif obj.orientation==2 % yz
%                         obj.img = permute(dataset,[1 4 3 2]);
%                     end
%                 end
%             else
%                 if permuteSw == 0 && obj.orientation==4 % yx
%                     obj.img(:,:,col_channel,:) = dataset;
%                 else    % get permuted dataset
%                     if obj.orientation==1     % xz
%                         obj.img(:,:,col_channel,:) = permute(dataset ,[4 1 3 2]);
%                     elseif obj.orientation==2 % yz
%                         obj.img(:,:,col_channel,:) = permute(dataset,[1 4 3 2]);
%                     end
%                 end
%             end
%         else    % set all other layers: model, mask, selection
%             if permuteSw == 1 && obj.orientation~=4 % yx    % permute dataset if needed
%                 if obj.orientation==1     % xz
%                     dataset = permute(dataset ,[3 1 2]);
%                 elseif obj.orientation==2 % yz
%                     dataset = permute(dataset,[1 3 2]);
%                 end
%             end
%             switch type
%                 case 'model'
%                     if ~isnan(col_channel)     % set only specific object
%                         obj.model(bitand(dataset, 1)==1) = bitand(obj.model(bitand(dataset, 1)==1), 192);  % 192 = 11000000, remove Material from the model
%                         obj.model(dataset==1) = bitor(obj.model(dataset==1), col_channel);
%                     else
%                         obj.model = bitand(obj.model, 192); % clear current model
%                         obj.model = bitor(obj.model, dataset);
%                     end
%                 case 'mask'
%                     obj.model = bitset(obj.model, 7, 0);    % clear current mask
%                     obj.model = bitor(obj.model, dataset*64);
%                     obj.maskExist = 1;
%                 case 'selection'
%                     obj.model = bitset(obj.model, 8, 0);    % clear existing selection
%                     obj.model = bitor(obj.model, dataset*128);
%                 case 'everything'
%                     obj.model = dataset;
%             end
%         end
%     else        % set the shown block
%         if isfield(options, 'y');
%             getDatDimOpt.blockModeSwitch = 0;
%             [yVal,xVal,~,zVal] = obj.getDatasetDimensions('image', NaN, NaN, getDatDimOpt);
%             Xlim = [max([options.x(1) 1]) min([options.x(2) xVal])];
%             Ylim = [max([options.y(1) 1]) min([options.y(2) yVal])];
%             Zlim = [max([options.z(1) 1]) min([options.z(2) zVal])];
%         else
%             if obj.orientation==1     % xz
%                 Xlim = ceil(obj.axesY);
%                 Ylim = [1 size(obj.img, obj.orientation)];
%                 Zlim = ceil(obj.axesX);
%             elseif obj.orientation==2 % yz
%                 Xlim = [1 size(obj.img, obj.orientation)];
%                 Ylim = ceil(obj.axesY);
%                 Zlim = ceil(obj.axesX);
%             elseif obj.orientation==4 % yx
%                 Xlim = ceil(obj.axesX);
%                 Ylim = ceil(obj.axesY);
%                 Zlim = [1 size(obj.img, obj.orientation)];
%             end
%             Xlim = [max([Xlim(1) 1]) min([Xlim(2) obj.width])];
%             Ylim = [max([Ylim(1) 1]) min([Ylim(2) obj.height])];
%             Zlim = [max([Zlim(1) 1]) min([Zlim(2) obj.no_stacks])];
%         end
%         if strcmp(type,'image')
%             if permuteSw == 1   % permute dataset
%                 if obj.orientation==1     % xz
%                     dataset = permute(dataset,[4 1 3 2]);
%                 elseif obj.orientation==2 % yz
%                     dataset = permute(dataset,[1 4 3 2]);
%                 end
%             end
%             obj.img(Ylim(1):Ylim(2),Xlim(1):Xlim(2),col_channel,Zlim(1):Zlim(2)) = dataset;
%         else
%             if permuteSw == 1   % permute dataset
%                 if obj.orientation==1     % xz
%                     dataset = permute(dataset,[3 1 2]);
%                 elseif obj.orientation==2 % yz
%                     dataset = permute(dataset,[1 3 2]);
%                 end
%             end
%             
%             switch type
%                 case 'model'
%                     if ~isnan(col_channel)     % take only specific object
%                         currentDataset = obj.model(Ylim(1):Ylim(2),Xlim(1):Xlim(2),Zlim(1):Zlim(2));
%                         currentDataset(bitand(currentDataset, col_channel)==col_channel) = bitand(currentDataset(bitand(currentDataset, col_channel)==col_channel), 192);  % 192 = 11000000, remove Material from the model
%                         currentDataset(dataset==1) = bitor(currentDataset(dataset==1), col_channel);
%                         dataset = currentDataset;
%                     end
%                     obj.model(Ylim(1):Ylim(2),Xlim(1):Xlim(2),Zlim(1):Zlim(2)) = dataset;
%                 case 'mask'
%                     currentDataset = obj.model(Ylim(1):Ylim(2),Xlim(1):Xlim(2),Zlim(1):Zlim(2));
%                     currentDataset = bitset(currentDataset,7,0);    % clear mask
%                     currentDataset = bitor(currentDataset, dataset*64);
%                     obj.model(Ylim(1):Ylim(2),Xlim(1):Xlim(2),Zlim(1):Zlim(2)) = currentDataset;
%                     obj.maskExist = 1;
%                 case 'selection'
%                     currentDataset = obj.model(Ylim(1):Ylim(2),Xlim(1):Xlim(2),Zlim(1):Zlim(2));
%                     currentDataset = bitset(currentDataset,8,0);    % clear selection
%                     currentDataset = bitor(currentDataset, dataset*128);
%                     obj.model(Ylim(1):Ylim(2),Xlim(1):Xlim(2),Zlim(1):Zlim(2)) = currentDataset;
%                 case 'everything'
%                     obj.model(Ylim(1):Ylim(2),Xlim(1):Xlim(2),Zlim(1):Zlim(2)) = dataset;
%             end
%         end
%     end
% end
% 
% % update obj.height, obj.width, etc
% if strcmp(type,'image') && options.blockModeSwitch == 0
%     obj.height = size(obj.img, 1);
%     obj.width = size(obj.img, 2);
%     obj.no_stacks = size(obj.img, 4);
%     obj.img_info('Height') = size(obj.img, 1);
%     obj.img_info('Width') = size(obj.img, 2);
%     obj.img_info('Stacks') = size(obj.img, 4);
%     
%     % update I.slices
%     currSlices = obj.slices;
%     obj.slices = [1, obj.height; 1, obj.width; 1, size(obj.img,3); 1,size(obj.no_stacks,4)];
%     
%     if currSlices(obj.orientation) > size(obj.img, obj.orientation)
%         obj.slices(obj.orientation,:) = size(obj.img, obj.orientation);
%     else
%         obj.slices(obj.orientation,:) = currSlices(obj.orientation,:);
%     end
%     
%     obj.current_yxz(1) = min([obj.current_yxz(1) obj.height]);
%     obj.current_yxz(2) = min([obj.current_yxz(2) obj.width]);
%     obj.current_yxz(3) = min([obj.current_yxz(3) obj.no_stacks]);
% end
end