function handles = ib_do_backup(handles, type, switch3d)
% function handles = ib_do_backup(handles, type, switch3d)
% Store the dataset for Undo
%
% The dataset is stored in imageUndo class
% 
% Parameters:
% handles: structure with handles of im_browser.m
% type: ''image'', ''selection'', ''mask'', ''model'', 'labels', ''everything'' (for imageData.model_type==''uint6'' only)
% switch3d: - a switch to define a 2D or 3D mode to store the dataset dataset
% - @b 0 - 2D slice
% - @b 1 - 3D dataset

% Copyright (C) 19.05.2014, Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 03.09.2015, Ilya Belevich, updated to use imageData.getData4D methods

if strcmp(handles.preferences.undo, 'no'); return; end;

if strcmp(handles.Img{handles.Id}.I.model_type, 'uint6')
    if strcmp(type, 'selection') || strcmp(type, 'mask') || strcmp(type, 'model')
        type = 'everything';
    end
end

if strcmp(type, 'mask') && isnan(handles.Img{handles.Id}.I.maskImg(1))
    return;
end;

if switch3d == 1        % 3D mode
    storeOptions.blockModeSwitch=0;
    if strcmp(type, 'image')
        handles.U.store(type, handles.Img{handles.Id}.I.getData4D(type, NaN, 4, 0, storeOptions), handles.Img{handles.Id}.I.img_info);
    elseif strcmp(type, 'labels')
        [labels.labelText, labels.labelPosition] = handles.Img{handles.Id}.I.hLabels.getLabels();
        handles.U.store(type, labels, NaN, 4, 1);
    elseif strcmp(type, 'measurements')
        handles.U.store(type, handles.Img{handles.Id}.I.hMeasure.Data, NaN, 4, 1);
    else
        if strcmp(handles.preferences.disableSelection, 'yes'); return; end;    % do not make backups if selection is disabled
        handles.U.store(type, handles.Img{handles.Id}.I.getData4D(type, NaN, 4, NaN, storeOptions));
    end
else                    % 2D mode
    if strcmp(type, 'image')
        handles.U.store(type, handles.Img{handles.Id}.I.getFullSlice(type, NaN, NaN, 0), handles.Img{handles.Id}.I.img_info, handles.Img{handles.Id}.I.orientation, handles.Img{handles.Id}.I.getCurrentSliceNumber());
    elseif strcmp(type, 'labels')
        [labels.labelText, labels.labelPosition] = handles.Img{handles.Id}.I.hLabels.getLabels();
        handles.U.store(type, labels, NaN, 4, 1);
    elseif strcmp(type, 'measurements')
        handles.U.store(type, handles.Img{handles.Id}.I.hMeasure.Data, NaN, 4, 1);
    else
        if strcmp(handles.preferences.disableSelection, 'yes'); return; end;    % do not make backups if selection is disabled
        handles.U.store(type, handles.Img{handles.Id}.I.getFullSlice(type), NaN, handles.Img{handles.Id}.I.orientation, handles.Img{handles.Id}.I.getCurrentSliceNumber());
    end
end
%sprintf('Backup: Index=%d, numel=%d, max=%d', handles.U.undoIndex, numel(handles.U.undoList), handles.U.max_steps)
end