function imageFilterDoitBtn_Callback(hObject, eventdata, handles)
% function imageFilterDoitBtn_Callback(hObject, eventdata, handles)
% a callback to the handles.imageFilterDoitBtn, filters image with the selected filter
%
% Parameters:
% hObject: handle to im_browser.m (see GCBO)
% eventdata: eventdata structure 
% handles: structure with handles of im_browser.m

% Copyright (C) 14.05.2014, Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 03.09.2015, Ilya Belevich, updated to use imageData.getData4D methods in ib_getDataset

filter_val = get(handles.imageFilterPopup,'Value');
filter_list = get(handles.imageFilterPopup,'String');

switch filter_list{filter_val}
    case 'Edge Enhancing Coherence Filter'
        handles = ib_anisotropicDiffusion(handles, 'coherence_filter');
        handles.Img{handles.Id}.I.plotImage(handles.imageAxes, handles, 0);
        return;
    case 'Perona Malik anisotropic diffusion'
        handles = ib_anisotropicDiffusion(handles, 'anisodiff');
        handles.Img{handles.Id}.I.plotImage(handles.imageAxes, handles, 0);
        return;
    case {'Diplib: Perona Malik anisotropic diffusion','Diplib: Robust Anisotropic Diffusion','Diplib: Mean Curvature Diffusion',...
            'Diplib: Corner Preserving Diffusion','Diplib: Kuwahara filter for edge-preserving smoothing'}
        handles = ib_anisotropicDiffusion(handles, 'diplib');
        handles.Img{handles.Id}.I.plotImage(handles.imageAxes, handles, 0);
        return;
end

hsize_txt = get(handles.imfiltPar1Edit,'String');
semicolon = strfind(hsize_txt,';');
if ~isempty(semicolon)
    hsize(1) = str2double(hsize_txt(1:semicolon(1)-1));
    hsize(2) = str2double(hsize_txt(semicolon(1)+1:end));
else
    hsize(1) = str2double(hsize_txt);
    hsize(2) = hsize(1);
end

options.dataType = '4D';
options.fitType = cell2mat(filter_list(filter_val));

%options.colorChannel = get(handles.ColChannelCombo,'Value')-1;
if numel(handles.Img{handles.Id}.I.slicesColor) ~= 1    % get color channel from the selected in the Selection panel
    options.colorChannel = get(handles.ColChannelCombo,'Value') - 1;
else    % when only one color channel is shown, take it
    options.colorChannel = handles.Img{handles.Id}.I.slicesColor;
end

if strcmp(get(handles.imfiltPar1Edit,'enable'), 'on')
    options.hSize = hsize;
end
if strcmp(get(handles.imfiltPar2Edit,'Enable'), 'on'); options.sigma = str2double(get(handles.imfiltPar2Edit,'String'));  end;
if strcmp(get(handles.imfiltPar3Edit,'Enable'), 'on'); options.lambda = str2double(get(handles.imfiltPar3Edit,'String')); end;
if strcmp(options.fitType,'Gaussian 3d')       % 3d gaussian filter
    if numel(hsize) == 1; handles.Img{handles.Id}.I.updateParameters(); end; % requires info about voxel sizes
end

options.pixSize = handles.Img{handles.Id}.I.pixSize;    % for 3D gaussian filter
if get(handles.roiShowCheck, 'value')   % when ROI mode is on, the returned dataset is transposed
    options.orientation = 4;
else
    options.orientation = handles.Img{handles.Id}.I.orientation;
end

if get(handles.imageFilterAllCheck,'value')
    ib_do_backup(handles, 'image', 1);
    img = ib_getDataset('image', handles, NaN, options.colorChannel);
else
    ib_do_backup(handles, 'image', 0);
    img = ib_getSlice('image', handles);
end
for roi=1:numel(img)
    [img{roi}, log_text] = ib_doImageFiltering(img{roi}, options);
end
if get(handles.imageFilterAllCheck,'value')
    ib_setDataset('image', img, handles, NaN, options.colorChannel);
else
    ib_setSlice('image', img, handles);
    log_text = [log_text ',slice=' num2str(handles.Img{handles.Id}.I.getCurrentSliceNumber())];
end

if isnan(log_text); return; end;
handles.Img{handles.Id}.I.updateImgInfo(log_text);
handles.Img{handles.Id}.I.plotImage(handles.imageAxes, handles, 0);
end