function loadModelBtn_Callback(hObject, eventdata, handles)
% function loadModelBtn_Callback(hObject, eventdata, handles)
% a callback to the handles.loadModelBtn, loads model to MIB from
% a file
%
% Parameters:
% hObject: handle to im_browser.m (see GCBO)
% eventdata: eventdata structure 
% handles: structure with handles of im_browser.m

% Copyright (C) 14.05.2014, Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 03.09.2015 Ilya Belevich, updated to use imageData.setData5D method

% load Model from a file
unFocus(hObject);   % remove focus from hObject
[filename, path] = uigetfile(...
    {'*.mat;',  'Matlab format (*.mat)'; ...
    '*.am;',  'Amira mesh format (*.am)'; ...
    '*.nrrd;',  'NRRD format (*.nrrd)'; ...
    '*.tif;',  'TIF format (*.tif)'; ...
    '*.*',  'All Files (*.*)'}, ...
    'Open model data...',handles.mypath,'MultiSelect','on');
if isequal(filename, 0); return; end; % check for cancel
tic
if ischar(filename); filename = cellstr(filename); end;     % convert to cell type
wb = waitbar(0,sprintf('%s\nPlease wait...',filename{1}),'Name','Loading model','WindowStyle','modal');
set(findall(wb,'type','text'),'Interpreter','none');
waitbar(0, wb);

createModelBtn_Callback(handles.createModelBtn,eventdata, handles); % make an empty model
if strcmp(filename{1}(end-2:end),'mat') % loading model in matlab format 
    res = load([path filename{1}]);
    handles.Img{handles.Id}.I.model_fn = fullfile([path filename{1}]);
    Fields = fieldnames(res);
    for field = 1:numel(Fields)     % find field name for the data
        if ~strcmp(Fields{field},'material_list') && ~strcmp(Fields{field},'labelText') && ~strcmp(Fields{field},'labelPosition') && ~strcmp(Fields{field},'color_list') && ~strcmp(Fields{field},'bounding_box')
            handles.Img{handles.Id}.I.model_var = Fields{field};
        end
    end

    model = res.(handles.Img{handles.Id}.I.model_var);
    if isfield(res, 'material_list')
        material_list = res.material_list;
    else    % was using permute earlier to to match height and width with the image
        model = permute(model,[2 1 3]); %#ok<NODEF>
    end
    if isfield(res, 'color_list')
        color_list = res.color_list; %#ok<NASGU>
    end
    
    % fill labels
    if isfield(res, 'labelText')
        handles.Img{handles.Id}.I.hLabels.addLabels(res.labelText, res.labelPosition);
    end
elseif strcmp(filename{1}(end-1:end),'am') % loading amira mesh
    model = amiraLabels2bitmap(fullfile([path filename{1}]));
    handles.Img{handles.Id}.I.model_fn = fullfile([path filename{1}(1:end-2) 'mat']);
    handles.Img{handles.Id}.I.model_var = 'amira_mesh';
elseif strcmp(filename{1}(end-3:end),'nrrd') % loading model in tif format  
    model = nrrdLoadWithMetadata(fullfile([path filename{1}]));
    model =  uint8(permute(model.data, [2 1 3]));
    handles.Img{handles.Id}.I.model_fn = fullfile([path filename{1}(1:end-2) 'mat']);
    handles.Img{handles.Id}.I.model_var = 'nrrd_model';
else        % loading model in tif format and other standard formats 
    for i=1:numel(filename)
        filename(i) = cellstr(fullfile([path filename{i}]));
    end;
    options.bioformatsCheck = 0;
    options.progressDlg = 0;
    [model, ~, ~] = ib_loadImages(filename, options, handles);
    model =  squeeze(model);
    handles.Img{handles.Id}.I.model_fn = fullfile([filename{1}(1:end-3) 'mat']);
    handles.Img{handles.Id}.I.model_var = 'tif_model';
end

if size(model,1) ~= size(handles.Img{handles.Id}.I.img,1) || size(model,2) ~= size(handles.Img{handles.Id}.I.img,2) || size(model,3) ~= size(handles.Img{handles.Id}.I.img,4)
    delete(wb);
    msgbox(sprintf('!!! ERROR !!!\n\nThe dimensions of the model (%d x %d x %d)\ndo not match with dimensions\n of the open dataset (%d x %d x %d)!', ...
        size(model,2), size(model,1), size(model,3), size(handles.Img{handles.Id}.I.img,2), size(handles.Img{handles.Id}.I.img,1), size(handles.Img{handles.Id}.I.img,4)),'Dimensions mismatch!','error','modal');
    return;
end
if handles.preferences.uint8
    handles.Img{handles.Id}.I.model_type = 'uint8';
else
    handles.Img{handles.Id}.I.model_type = 'uint6';
end

waitbar(0.5,wb);
options.blockModeSwitch = 0;
handles.Img{handles.Id}.I.setData5D('model', model, 4, NaN, options);
                                    
if exist('material_list', 'var')
    handles.Img{handles.Id}.I.modelMaterialNames = material_list;
else
    max_color = max(max(max(model)));
    if max_color > 0
        for i=1:max_color
            handles.Img{handles.Id}.I.modelMaterialNames(i,1) = cellstr(num2str(i));
        end
    end
end
updateSegmentationLists(handles);

% update colors for the materials
if exist('color_list', 'var')
    handles.Img{handles.Id}.I.modelMaterialColors = res.color_list;
end

handles.lastSegmSelection = 1;
waitbar(1,wb);
modelShowCheck_Callback(handles.modelShowCheck, eventdata, handles);

delete(wb);
toc
% do not put guidata here! guidata(handles.im_browser, handles);
end