function menuDatasetSlice(hObject, eventdata, handles, parameter)
% function menuDatasetSlice(hObject, eventdata, handles, parameter)
% a callback to Menu->Dataset->Slice
% do actions with individual slices
%
% Parameters:
% hObject: handle to im_browser.m (see GCBO)
% eventdata: eventdata structure 
% handles: structure with handles of im_browser.m
% parameter: a string that defines image source:
% - 'delete', delete slice from the dataset

% Copyright (C) 14.05.2014, Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% part of Microscopy Image Browser, http:\\mib.helsinki.fi 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 


currentSlice = handles.Img{handles.Id}.I.getCurrentSliceNumber();
maxSlice = size(handles.Img{handles.Id}.I.img, handles.Img{handles.Id}.I.orientation);

switch parameter
    case 'delete'
        %answer=inputdlg(sprintf('Please enter slice number(s) to delete (1:%d):',maxSlice),'Enter slice number',1,{num2str(currentSlice)});
        answer=mib_inputdlg(NaN,sprintf('Please enter slice number(s) to delete (1:%d):',maxSlice),'Enter slice number',num2str(currentSlice));
        if isempty(answer); return; end;

        result = handles.Img{handles.Id}.I.deleteSlice(str2num(answer{1})); %#ok<ST2NM>
        if result == 0; return; end;
        % generate the log text
        log_text = sprintf('Delete slice: %s, Orient: %d', answer{1}, handles.Img{handles.Id}.I.orientation);
end

handles.Img{handles.Id}.I.updateImgInfo(log_text);
% update widgets in the im_browser GUI
handles = updateGuiWidgets(handles);
% redraw image in the im_browser axes
handles.Img{handles.Id}.I.plotImage(handles.imageAxes, handles, 0);
end