function menuFileSaveImageAs(hObject, eventdata, handles)
% function menuFileSaveImageAs(hObject, eventdata, handles)
% a callback to the handles.menuFileSaveImageAs, saves image to a file
%
% Parameters:
% hObject: handle to im_browser.m (see GCBO)
% eventdata: eventdata structure 
% handles: structure with handles of im_browser.m

% Copyright (C) 14.05.2014, Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% part of Microscopy Image Browser, http:\\mib.helsinki.fi 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 


% save image as...
if size(handles.Img{handles.Id}.I.img,1)<1; msgbox('No image detected!','Error!','error','modal'); return; end;

fn_out = handles.Img{handles.Id}.I.img_info('Filename');
if isempty(strfind(fn_out,'/')) && isempty(strfind(fn_out,'\'))
    fn_out = fullfile(handles.mypath, fn_out);
end
if isempty(fn_out)
    fn_out = handles.mypath;
end

Filters = {'*.am;',  'Amira Mesh binary (*.am)';...
           '*.jpg;',  'Joint Photographic Experts Group (*.jpg)'; ...
           '*.h5',   'Hierarchical Data Format (*.h5 *.hdf)'; ...
           '*.mrc;',  'MRC format for IMOD (*.mrc)'; ...
           '*.nrrd',  'NRRD Data Format (*.nrrd)'; ...
           '*.png',   'Portable Network Graphics (*.png)'; ...
           '*.tif;',  'TIF format LZW compression (*.tif)'; ...
           '*.tif;',  'TIF format uncompressed (*.tif)'; ...
           '*.*',  'All Files (*.*)'};

[pathStr,fnameStr,ext] = fileparts(fn_out);
if strcmp('.am', ext)
    %Filters = Filters([6 1:5 7:end],:);
elseif strcmp('.jpg', ext)
    Filters = Filters([2 1 3:end],:);
    fn_out = fullfile(pathStr, [fnameStr '.jpg']);    
elseif strcmp('.h5', ext)    
    Filters = Filters([3 1:2 4:end],:);
elseif strcmp('.mrc', ext)     % Volume for IMOD (*.mrc)
    Filters = Filters([4 1:3 5:end],:);
elseif strcmp('.nrrd', ext)
    Filters = Filters([5 1:4 6:end],:);
elseif strcmp('.png', ext)
    Filters = Filters([6 1:5 7:end],:);
elseif strcmp('.tif', ext)
    Filters = Filters([7 1:6 8:end],:);
    fn_out = fullfile(pathStr, [fnameStr '.tif']);
else
    Filters = Filters([7 1:6 8:end],:);
end

[filename, path, FilterIndex] = uiputfile(Filters, 'Save image...',fn_out); %...
if isequal(filename,0); return; end; % check for cancel
if strcmp('Amira Mesh binary (*.am)', Filters{FilterIndex,2})    % am format
    handles.Img{handles.Id}.I.updateParameters();
    pause(.1);
    tic;
    savingOptions = struct('overwrite', 1);
    savingOptions.colors = handles.Img{handles.Id}.I.lutColors;   % store colors for color channels 0-1;
    %bitmap2amiraMesh(fullfile(path, filename), handles.Img{handles.Id}.I.img, handles.Img{handles.Id}.I.img_info, savingOptions);
    bitmap2amiraMesh(fullfile(path, filename), handles.Img{handles.Id}.I.img, ...
         containers.Map(keys(handles.Img{handles.Id}.I.img_info),values(handles.Img{handles.Id}.I.img_info)), savingOptions);
elseif strcmp('Hierarchical Data Format (*.h5 *.hdf)', Filters{FilterIndex,2})     % hdf5 format
    handles.Img{handles.Id}.I.updateParameters();
    pause(.1);
    tic;
    savingOptions = struct('overwrite', 1);
    image2hdf5(fullfile(path, filename), handles.Img{handles.Id}.I.img, savingOptions, ...
        containers.Map(keys(handles.Img{handles.Id}.I.img_info),values(handles.Img{handles.Id}.I.img_info)));
elseif strcmp('Joint Photographic Experts Group (*.jpg)', Filters{FilterIndex,2})    % jpg format
    tic;
    savingOptions = struct('overwrite', 1,'Comment', handles.Img{handles.Id}.I.img_info('ImageDescription'));
    if strcmp(handles.Img{handles.Id}.I.img_info('ColorType'), 'indexed')
        savingOptions.cmap = handles.Img{handles.Id}.I.img_info('Colormap');
    else
        savingOptions.cmap = NaN;
    end
    % get list of filenames for slices
    if isKey(handles.Img{handles.Id}.I.img_info, 'SliceName')
        savingOptions.SliceName = handles.Img{handles.Id}.I.img_info('SliceName');
    end
    ib_image2jpg(fullfile(path, filename), handles.Img{handles.Id}.I.img, savingOptions);
elseif strcmp('MRC format for IMOD (*.mrc)', Filters{FilterIndex,2})    % MRC format
    tic;
    savingOptions.volumeFilename = fullfile(path, filename);
    savingOptions.pixSize = handles.Img{handles.Id}.I.pixSize;
    ib_image2mrc(handles.Img{handles.Id}.I.img, savingOptions);
elseif strcmp('Portable Network Graphics (*.png)', Filters{FilterIndex,2})    % PNG format
    savingOptions = struct('overwrite', 1,'Comment', handles.Img{handles.Id}.I.img_info('ImageDescription'),...
        'XResolution', handles.Img{handles.Id}.I.img_info('XResolution'), 'YResolution', handles.Img{handles.Id}.I.img_info('YResolution'), ...
        'ResolutionUnit', 'Unknown', 'Reshape', 0);
    if strcmp(handles.Img{handles.Id}.I.img_info('ColorType'), 'indexed')
        savingOptions.cmap = handles.Img{handles.Id}.I.img_info('Colormap');
    else
        savingOptions.cmap = NaN;
    end
    % get list of filenames for slices
    if isKey(handles.Img{handles.Id}.I.img_info, 'SliceName')
        savingOptions.SliceName = handles.Img{handles.Id}.I.img_info('SliceName');
    end
    ib_image2png(fullfile(path, filename), handles.Img{handles.Id}.I.img, savingOptions);
elseif strcmp('NRRD Data Format (*.nrrd)', Filters{FilterIndex,2})    % PNG format
    tic;
    savingOptions = struct('overwrite', 1);
    bb = handles.Img{handles.Id}.I.getBoundingBox();
    bitmap2nrrd(fullfile(path, filename), handles.Img{handles.Id}.I.img, bb, savingOptions);
else    % tif format
    handles.Img{handles.Id}.I.updateParameters();
    pause(.1);
    tic;
    if strcmp('TIF format LZW compression (*.tif)', Filters{FilterIndex,2})
        compression = 'lzw';
    else
        compression = 'none';
    end
    colortype = handles.Img{handles.Id}.I.img_info('ColorType');
    if strcmp(colortype,'indexed')
        cmap = handles.Img{handles.Id}.I.img_info('Colormap');
    else
        cmap = NaN;
    end
    
    ImageDescription = {handles.Img{handles.Id}.I.img_info('ImageDescription')};
    savingOptions = struct('Resolution', [handles.Img{handles.Id}.I.img_info('XResolution') handles.Img{handles.Id}.I.img_info('YResolution')],...
        'overwrite', 1, 'Saving3d', NaN, 'cmap', cmap, 'Compression', compression);
    % get list of filenames for slices
    if isKey(handles.Img{handles.Id}.I.img_info, 'SliceName')
        savingOptions.SliceName = handles.Img{handles.Id}.I.img_info('SliceName');
    end
    
    if handles.Img{handles.Id}.I.no_stacks == 1
        savingOptions.Saving3d = 'multi';
    end
    ib_image2tiff(fullfile(path, filename), handles.Img{handles.Id}.I.img, savingOptions, ImageDescription);
end
update_filelist(handles, filename);     % update list of files, use filename to highlight the saved file
toc;
end