function menuMaskSaveAs(hObject, eventdata, handles)
% function menuMaskSaveAs(hObject, eventdata, handles)
% a callback to Menu->Mask->Save As
% saves the Mask layer to a file
%
% Parameters:
% hObject: handle to im_browser.m (see GCBO)
% eventdata: eventdata structure 
% handles: structure with handles of im_browser.m

% Copyright (C) 14.05.2014, Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% part of Microscopy Image Browser, http:\\mib.helsinki.fi 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 07.09.2015, IB updated to use imageData.getData4D methods

% Save mask layer to a file in Matlab format
if handles.Img{handles.Id}.I.maskExist == 0; disp('Cancelled: No mask information found!'); return; end;
if isnan(handles.Img{handles.Id}.I.maskImgFilename)
    [pathstr, name] = fileparts(handles.Img{handles.Id}.I.img_info('Filename'));
    fn_out = fullfile(pathstr, [name '.mask']);
    if isempty(strfind(fn_out,'/')) && isempty(strfind(fn_out,'\'))
        fn_out = fullfile(handles.mypath, fn_out);
    end
    if isempty(fn_out)
        fn_out = handles.mypath;
    end
else
    fn_out = handles.Img{handles.Id}.I.maskImgFilename;
end;
[filename, path, FilterIndex] = uiputfile(...
    {'*.mask',  'Matlab format (*.mask)'; ...
    '*.am',  'Amira mesh binary RLE compression SLOW (*.am)'; ...
    '*.am',  'Amira mesh binary (*.am)'; ...
    '*.tif',  'TIF format (*.tif)'; ...
    '*.*',  'All Files (*.*)'}, ...
    'Save mask data...', fn_out);
if isequal(filename,0); return; end; % check for cancel
%warning('off','MATLAB:gui:latexsup:UnableToInterpretTeXString');    % switch off warnings for latex
curInt = get(0, 'DefaulttextInterpreter'); 
set(0, 'DefaulttextInterpreter', 'none'); 
wb = waitbar(0,sprintf('%s\nPlease wait...',fullfile(path, filename)),'Name','Saving the mask','WindowStyle','modal');

getDataOptions.blockModeSwitch = 0;
imOut = handles.Img{handles.Id}.I.getData4D('mask', NaN, 4, 0, getDataOptions);
waitbar(0.1, wb);

if FilterIndex == 1     % matlab file
    save([path filename], 'imOut','-v7.3');
elseif FilterIndex == 2 || FilterIndex == 3      % Amira mesh
    if FilterIndex == 2
        amiraType = 'binaryRLE';
    else
        amiraType = 'binary';
    end
    bb = handles.Img{handles.Id}.I.getBoundingBox();
    pixStr = handles.Img{handles.Id}.I.pixSize;
    pixStr.minx = bb(1);
    pixStr.miny = bb(3);
    pixStr.minz = bb(5);
    bitmap2amiraLabels([path filename], imOut, amiraType, pixStr, handles.preferences.maskcolor,cellstr('Mask'), 1);
elseif FilterIndex == 4   % as tif
    ImageDescription = {handles.Img{handles.Id}.I.img_info('ImageDescription')};
    resolution(1) = handles.Img{handles.Id}.I.img_info('XResolution');
    resolution(2) = handles.Img{handles.Id}.I.img_info('YResolution');
    savingOptions = struct('Resolution', resolution, 'overwrite', 1, 'Saving3d', NaN, 'cmap', NaN);
    ib_image2tiff(fullfile(path, filename), imOut, savingOptions, ImageDescription);
end
waitbar(0.9, wb);
handles.Img{handles.Id}.I.maskImgFilename = fullfile(path, filename);
sprintf('The mask %s was saved!', fullfile(path, filename));
waitbar(1, wb);
set(0, 'DefaulttextInterpreter', curInt); 
delete(wb);
end