function menuModelsImport(hObject, eventdata, handles)
% function menuModelsImport(~, eventdata, handles)
% a callback to Menu->Models->Import
% imports the Model layer from the main Matlab workspace
%
% Parameters:
% hObject: handle to im_browser.m (see GCBO)
% eventdata: eventdata structure 
% handles: structure with handles of im_browser.m

% Copyright (C) 14.05.2014, Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% part of Microscopy Image Browser, http:\\mib.helsinki.fi 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 


% do nothing is selection is disabled
if strcmp(handles.preferences.disableSelection, 'yes'); 
    warndlg(sprintf('The model layer is switched off!\n\nPlease make sure that the "Disable selection" option in the Preferences dialog (Menu->File->Preferences) is set to "no" and try again...'),'The models are disabled','modal');
    return; 
end;

prompt = sprintf('Enter the name of the model variable.\nIt may be a matrix (1:height,1:width,1:z)\nor a structure with "model" and "materials" fields');
title = 'Import from Matlab';
%answer = inputdlg(prompt,title,1,{'O'},'on');
answer = mib_inputdlg(NaN,prompt,title,'O');
if size(answer) == 0; return; end;

if (~isempty(answer{1}))
    try
        varIn = evalin('base',answer{1});
    catch exception
        errordlg(sprintf('The variable was not found in the Matlab base workspace:\n\n%s', exception.message),'Misssing variable!','modal');
        return;
    end

    if isstruct(varIn)
        model = varIn.model;
        if isfield(varIn, 'materials'); material_list = varIn.materials; end;
        if isfield(varIn, 'colors'); material_colors = varIn.colors; end;
    else
        model = varIn;
    end
    %model = permute(model,[2 1 3]);
    if size(model,1) ~= size(handles.Img{handles.Id}.I.img,1) || size(model,2) ~= size(handles.Img{handles.Id}.I.img,2)
        msgbox(sprintf('Model and image dimensions mismatch!\nImage (HxW) = %d x %d pixels\nModel (HxW) = %d x %d pixels',...
            size(handles.Img{handles.Id}.I.img,1),size(handles.Img{handles.Id}.I.img,2),size(model,1),size(model,2)),'Error!','error','modal');
        return;
    end
    if size(model,3) == 1
        if handles.Img{handles.Id}.I.modelExist == 0
            createModelBtn_Callback(handles.createModelBtn,eventdata, handles);
        end
        handles.Img{handles.Id}.I.setCurrentSlice('model', model);
    elseif size(model,3) == size(handles.Img{handles.Id}.I.img,4)
        createModelBtn_Callback(handles.createModelBtn,eventdata, handles);
        options.blockModeSwitch = 0;
        handles.Img{handles.Id}.I.setData4D('model', model, NaN, 4, NaN, options);
    end
    if exist('material_list', 'var')
        handles.Img{handles.Id}.I.modelMaterialNames = material_list;
        max_color = numel(material_list);
    else
        max_color = max(max(max(model)));
        if max_color > 0
            for i=1:max_color
                handles.Img{handles.Id}.I.modelMaterialNames(i,1) = cellstr(num2str(i));
            end
        end
    end
    if exist('material_list', 'var')
        handles.Img{handles.Id}.I.modelMaterialColors = material_colors;
    end
    
    % adding extra colors if needed
    if max_color > size(handles.Img{handles.Id}.I.modelMaterialColors,1)
        minId = size(handles.Img{handles.Id}.I.modelMaterialColors,1)+1;
        maxId = max_color;
        handles.Img{handles.Id}.I.modelMaterialColors = [handles.Img{handles.Id}.I.modelMaterialColors; rand([maxId-minId+1,3])];
    end
    
    % add annotations
    if isstruct(varIn)
        if isfield(varIn, 'labelText');
            handles.Img{handles.Id}.I.hLabels.addLabels(varIn.labelText, varIn.labelPosition);
        end;
    end;
end
updateSegmentationLists(handles);
modelShowCheck_Callback(handles.modelShowCheck,eventdata,handles);
handles.Img{handles.Id}.I.plotImage(handles.imageAxes, handles, 0);
end
