function menuModelsSaveAs(hObject, eventdata, handles)
% function menuModelsSaveAs(hObject, eventdata, handles)
% a callback to Menu->Models->Save as
% saves model to a file
%
% Parameters:
% hObject: handle to im_browser.m (see GCBO)
% eventdata: eventdata structure 
% handles: structure with handles of im_browser.m

% Copyright (C) 14.05.2014, Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% part of Microscopy Image Browser, http:\\mib.helsinki.fi 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 04.09.2015, Ilya Belevich, adapted to use getData4D method
% 08.01.2016, Ilya Belevich, added export using the STL format

if  handles.Img{handles.Id}.I.modelExist == 0 || strcmp(handles.Img{handles.Id}.I.model_type, 'int8'); disp('Cancel: No segmentation model detected'); return; end;
if isnan(handles.Img{handles.Id}.I.model(1,1,1)); errordlg('No model found!','Error!'); return; end;
fn_out = handles.Img{handles.Id}.I.model_fn;
contour = get(handles.segmList,'Value');
if get(handles.seeAllMaterialsCheck, 'value');     contour = 0;  end;
if isempty(fn_out)
    fn_out = handles.mypath;
end
[filename, path, FilterIndex] = uiputfile(...
    {'*.mat;',  'Matlab format (*.mat)'; ...
    '*.tif;',  'TIF format (*.tif)'; ...
    '*.am;',  'Amira mesh binary RLE compression SLOW (*.am)'; ...
    '*.am;',  'Amira mesh binary (*.am)'; ...
    '*.am;',  'Amira mesh ascii (*.am)'; ...
    '*.mod;',  'Contours for IMOD (*.mod)'; ...
    '*.mrc;',  'Volume for IMOD (*.mrc)'; ...
    '*.nrrd;',  'NRRD for 3D Slicer (*.nrrd)'; ...
    '*.stl',  'Isosurface as binary STL (*.stl)'; ...
    '*.*',  'All Files (*.*)'}, ...
    'Save model data...',fn_out);
if isequal(filename,0); return; end; % check for cancel
tic

getDataOptions.blockModeSwitch = 0;     % get the full dataset

if FilterIndex == 1     % matlab file
    %model = permute(handles.Img{handles.Id}.I.model,[2 1 3]); %#ok<NASGU>  % earlier the models were permuted
    %str1 = strcat(handles.Img{handles.Id}.I.model_var, '=model;');
    warning('off','MATLAB:gui:latexsup:UnableToInterpretTeXString');    % switch off warnings for latex
	curInt = get(0, 'DefaulttextInterpreter'); 
	set(0, 'DefaulttextInterpreter', 'none');
    wb = waitbar(0,sprintf('%s\nPlease wait...',fullfile(path, filename)),'Name','Saving the model','WindowStyle','modal');
    set(findall(wb,'type','text'),'Interpreter','none');
    waitbar(0, wb);
    str1 = strcat(handles.Img{handles.Id}.I.model_var, '=handles.Img{handles.Id}.I.getData4D(''model'', NaN, 4, NaN, getDataOptions);');
    eval(str1);
    material_list = handles.Img{handles.Id}.I.modelMaterialNames; %#ok<NASGU>
    color_list = handles.Img{handles.Id}.I.modelMaterialColors; %#ok<NASGU>
    bounding_box = handles.Img{handles.Id}.I.getBoundingBox(); %#ok<NASGU>
    if handles.Img{handles.Id}.I.hLabels.getLabelsNumber() > 1  % save annotations
        [labelText, labelPosition] = handles.Img{handles.Id}.I.hLabels.getLabels(); %#ok<NASGU,ASGLU>
        str1 = ['save ''' fullfile(path, filename) ''' ' handles.Img{handles.Id}.I.model_var ' material_list color_list bounding_box labelText labelPosition -mat -v7.3'];
    else    % save without annotations
        str1 = ['save ''' fullfile(path, filename) ''' ' handles.Img{handles.Id}.I.model_var ' material_list color_list bounding_box -mat -v7.3'];
    end
    eval(str1);
    handles.Img{handles.Id}.I.model_fn = fullfile(path, filename);
    delete(wb);
	set(0, 'DefaulttextInterpreter', curInt);
else
    if contour > 0
        %button = questdlg(sprintf('You are going to export only countour No:%d !\nProceed?',contour-1),'Single contour export','Proceed','Cancel','Proceed');
        button = questdlg(sprintf('You are going to export only material No:%d (%s) !\nProceed?', contour, handles.Img{handles.Id}.I.modelMaterialNames{contour}),'Single contour export','Proceed, set as 1','Proceed, set as 255','Cancel','Proceed, set as 1');
        if strcmp(button, 'Cancel'); return; end;
        model = handles.Img{handles.Id}.I.getData4D('model', NaN, 4, contour, getDataOptions);
        if strcmp(button, 'Proceed, set as 255') && FilterIndex ~= 9; % assign index 255 to the material
            model = model*255;
        end
        color_list = handles.Img{handles.Id}.I.modelMaterialColors(contour,:);
        modelMaterialNames = handles.Img{handles.Id}.I.modelMaterialNames(contour);
    else
        model = handles.Img{handles.Id}.I.getData4D('model', NaN, 4, NaN, getDataOptions);
        color_list = handles.Img{handles.Id}.I.modelMaterialColors;
        color_list = color_list(1:numel(handles.Img{handles.Id}.I.modelMaterialNames),:);
        modelMaterialNames = handles.Img{handles.Id}.I.modelMaterialNames;
    end;
    
    if FilterIndex == 2     % as tif
        ImageDescription = {handles.Img{handles.Id}.I.img_info('ImageDescription')};
        resolution(1) = handles.Img{handles.Id}.I.img_info('XResolution');
        resolution(2) = handles.Img{handles.Id}.I.img_info('YResolution');
        savingOptions = struct('Resolution', resolution, 'overwrite', 1, 'Saving3d', NaN, 'cmap', NaN);
        model = reshape(model,[size(model,1) size(model,2) 1 size(model,3)]);
        ib_image2tiff(fullfile(path, filename), model, savingOptions, ImageDescription);
    elseif FilterIndex == 3     % Amira mesh binary RLE compression
        bb = handles.Img{handles.Id}.I.getBoundingBox();
        pixStr = handles.Img{handles.Id}.I.pixSize;
        pixStr.minx = bb(1);
        pixStr.miny = bb(3);
        pixStr.minz = bb(5);
        bitmap2amiraLabels(fullfile(path, filename), model, 'binaryRLE', pixStr, color_list,modelMaterialNames, 1);
    elseif FilterIndex == 4     % Amira mesh binary
        bb = handles.Img{handles.Id}.I.getBoundingBox();
        pixStr = handles.Img{handles.Id}.I.pixSize;
        pixStr.minx = bb(1);
        pixStr.miny = bb(3);
        pixStr.minz = bb(5);
        bitmap2amiraLabels(fullfile(path, filename), model, 'binary', pixStr, color_list,modelMaterialNames, 1);
    elseif FilterIndex == 5     % Amira mesh ascii
        bb = handles.Img{handles.Id}.I.getBoundingBox();
        pixStr = handles.Img{handles.Id}.I.pixSize;
        pixStr.minx = bb(1);
        pixStr.miny = bb(3);
        pixStr.minz = bb(5);
        bitmap2amiraLabels(fullfile(path, filename), model, 'ascii', pixStr, color_list,modelMaterialNames, 1);
    elseif FilterIndex == 6     % Contours for IMOD (*.mod)
        prompt = {'Take each Nth point in contours ( > 0):','Show detected points in the selection layer [0-no, 1-yes]:'};
        dlg_title = 'Parameters';
        answer = inputdlg(prompt,dlg_title,1,{'5','0'});
        if size(answer) == 0; return; end;
        Options.modelFilename = [path filename];
        Options.pixSize = handles.Img{handles.Id}.I.pixSize;
        Options.xyScaleFactor = str2double(answer{1});
        Options.zScaleFactor = 1;
        Options.generateSelectionSw = str2double(answer{2});
        Options.colorList = color_list;
        if Options.generateSelectionSw
            [~, selection] = exportModelToImodModel(model, Options);
            handles.Img{handles.Id}.I.setData4D('selection',selection, NaN, 4, 0, getDataOptions);
            handles.Img{handles.Id}.I.plotImage(handles.imageAxes, handles, 0);
            disp(['Model: ' fullfile(path, filename) ' has been saved']);
            toc;
            return;
        else
            exportModelToImodModel(model, Options);
        end
    elseif FilterIndex == 7     % Volume for IMOD (*.mrc)
        Options.volumeFilename = fullfile(path, filename);
        Options.pixSize = handles.Img{handles.Id}.I.pixSize;
        ib_image2mrc(model, Options);
    elseif FilterIndex == 8     % NRRD for 3D Slicer (*.nrrd)
        bb = handles.Img{handles.Id}.I.getBoundingBox;
        Options.overwrite = 1;
        bitmap2nrrd(fullfile(path, filename), model, bb, Options);
    elseif FilterIndex == 9     % STL isosurface for Blinder (*.stl)
        bounding_box = handles.Img{handles.Id}.I.getBoundingBox();  % get bounding box
        if contour == 0
            p = ib_renderModel(model, contour, handles.Img{handles.Id}.I.pixSize, bounding_box, handles.Img{handles.Id}.I.modelMaterialColors);
            [~,fnMain] = fileparts(filename);
            for i=1:numel(p)
                fv = struct('faces', p(i).Faces, 'vertices', p(i).Vertices);
                stlwrite(fullfile(path, [sprintf('%s_%d', fnMain, i) '.stl']), fv, 'FaceColor', p(i).FaceColor*255);
            end
        else
            p = ib_renderModel(model, 1, handles.Img{handles.Id}.I.pixSize, bounding_box, color_list);
            fv.faces = p.Faces;
            fv.vertices = p.Vertices;
            stlwrite(fullfile(path, filename), fv, 'FaceColor', p.FaceColor*255);
        end
        
    end
end
disp(['Model: ' fullfile(path, filename) ' has been saved']);
guidata(handles.im_browser, handles);
toc;
end