function model_cm_Callback(hObject, eventdata, type)
% function model_cm_Callback(hObject, eventdata, type)
% a context menu to the to the handles.segmList (Materials list of the Segmentation panel), the menu is called
% with the right mouse button
%
% Parameters:
% hObject: handle to im_browser.m (see GCBO)
% eventdata: eventdata structure 
% handles: structure with handles of im_browser.m

% Copyright (C) 14.05.2014, Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% part of Microscopy Image Browser, http:\\mib.helsinki.fi 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 


if isstruct(type)   % call from the Models menu entry
    type = 'statistics';
end
handles = guidata(hObject);
if isempty(handles.Img{handles.Id}.I.modelMaterialNames); return; end;
contIndex = get(handles.segmList,'Value');
% if contIndex==0 && ~strcmp(type, 'isosurface') && ~strcmp(type, 'volumeFiji') && ~strcmp(type, 'smooth')
%     msgbox('Please select object in the Materials list');
%     return;
% end
switch type
    case 'rename'
        segmList = handles.Img{handles.Id}.I.modelMaterialNames;
        segmListValue = get(handles.segmList,'Value');
        %answer = inputdlg(sprintf('Please add a new name for this material:'),'Rename material',1,segmList(segmListValue));
        answer = mib_inputdlg(NaN, sprintf('Please add a new name for this material:'),'Rename material',segmList{segmListValue});
        if ~isempty(answer)
            segmList(segmListValue) = answer(1);
            %set(handles.segmList,'String', segmList);
            handles.Img{handles.Id}.I.modelMaterialNames = segmList;
            updateSegmentationLists(handles);
        end
    case 'set color'
        figTitle = ['Set color for ' handles.Img{handles.Id}.I.modelMaterialNames{contIndex}];
        c =  uisetcolor(handles.Img{handles.Id}.I.modelMaterialColors(contIndex,:),figTitle);
        if length(c) ~= 1
            handles.Img{handles.Id}.I.modelMaterialColors(contIndex,:) = c;
        end;
    case 'smooth'
        smoothImage_Callback(NaN, NaN, handles, 'Model');
    case 'statistics'
        windowList = findall(0,'Type','figure');
        winStarted = 0;
        for i=1:numel(windowList) % re-initialize the window with keeping existing settings
            if strcmp(get(windowList(i),'tag'),'maskStatsDlg') % update imAdjustment window
                handles = MaskStatsDlg(handles, 'Model', windowList(i));
                winStarted = 1;
            end
        end
        if winStarted == 0  % re-initialize the window completely
            handles = MaskStatsDlg(handles,'Model');    
        end
        guidata(handles.im_browser, handles);
    case 'isosurface'
        options.fillBg = 0;
        model = ib_getDataset('model', handles, NaN, NaN, options);
        if numel(model) > 1;
            msgbox(sprintf('!!! Error !!!\nPlease select which of ROIs you would like to render!'),'Error!','error');
            return;
        end
        if get(handles.seeAllMaterialsCheck,'value') == 1; contIndex = 0; end;      % show all materials
        image = ib_getDataset('image', handles, NaN, NaN, options);
        bb = handles.Img{handles.Id}.I.getBoundingBox();  % get bounding box
        ib_renderModel(model{1}, contIndex, handles.Img{handles.Id}.I.pixSize, bb, handles.Img{handles.Id}.I.modelMaterialColors, image{1});
    case 'volumeFiji'
        options.fillBg = 0;
        model = ib_getDataset('model', handles, NaN, NaN, options);
        if numel(model) > 1;
            msgbox(sprintf('Error!\nPlease select a ROI to render!'),'Error!','error');
            return;
        end
        if get(handles.seeAllMaterialsCheck,'value') == 1; contIndex = 0; end;      % show all materials
        ib_renderModelFiji(model{1}, contIndex, handles.Img{handles.Id}.I.pixSize, handles.Img{handles.Id}.I.modelMaterialColors);
end
handles.Img{handles.Id}.I.plotImage(handles.imageAxes, handles, 0);
end