function selectionFillBtn_Callback(hObject, eventdata, handles, sel_switch)
% function selectionFillBtn_Callback(~, eventdata, handles, sel_switch)
% a callback to the handles.selectionFillBtn, allows to fill holes for the Selection layer
%
% Parameters:
% hObject: handle to im_browser.m (see GCBO)
% eventdata: eventdata structure 
% handles: structure with handles of im_browser.m
% sel_switch: a string that defines whether filling should be done for the
% current slice: 'current'; for all slices in 2d: 'all'

% Copyright (C) 14.05.2014, Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% part of Microscopy Image Browser, http:\\mib.helsinki.fi 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 


% do nothing is selection is disabled
if strcmp(handles.preferences.disableSelection, 'yes'); return; end;

selContour = get(handles.segmSelList,'Value')-2;
selectedOnly = get(handles.segmSelectedOnlyCheck,'Value');
if nargin < 4
    modifier = get(handles.im_browser,'currentModifier');
    if isempty(modifier)
        sel_switch = 'current';
    else
        sel_switch = 'all';
    end
end

if strcmp(sel_switch,'current')
    ib_do_backup(handles, 'selection', 0);
    filled_img = imfill(handles.Img{handles.Id}.I.getCurrentSlice('selection'),'holes');
    if selectedOnly
        filled_img = filled_img & handles.Img{handles.Id}.I.getCurrentSlice('model', selContour);
    end
    handles.Img{handles.Id}.I.setCurrentSlice('selection', filled_img);
elseif strcmp(sel_switch,'all') 
    wb = waitbar(0,'Filling holes in 2d for all slices...','WindowStyle','modal');
    ib_do_backup(handles, 'selection', 1);
    max_size = size(handles.Img{handles.Id}.I.img,handles.Img{handles.Id}.I.orientation);
    for layer_id=1:max_size
        if mod(layer_id, 10)==0; waitbar(layer_id/max_size, wb); end;
        slice = handles.Img{handles.Id}.I.getSlice('selection', layer_id);
        if max(max(slice)) < 1; continue; end;
        slice = imfill(slice,'holes');
        if selectedOnly
            slice = slice & handles.Img{handles.Id}.I.getCurrentSlice('model', selContour);
        end
        handles.Img{handles.Id}.I.setSlice('selection', slice, layer_id);
    end
    delete(wb);
end
handles.Img{handles.Id}.I.plotImage(handles.imageAxes, handles, 0);
end