function handles = ib_smoothImage(handles, type)
% function handles = ib_smoothImage(handles, type, options)
% Smooth 'Mask', 'Selection' or 'Model' layer
%
% Parameters:
% handles: structure with handles of im_browser.m
% type: a type of the layer for the smoothing:
% - ''selection'' - run size exclusion on the 'Selection' layer
% - ''model'' - - run size exclusion on the 'Model' layer
% - ''mask'' - - run size exclusion on the 'Mask' layer
% 
% Return values:
% handles: structure with handles of im_browser.m

% Copyright (C) 30.04.2014 Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% part of Microscopy Image Browser, http:\\mib.helsinki.fi 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 


title = [type 'Smoothing...'];
def = {'2d','5','5','5',};
prompt = {'Mode (''3d'' or ''2d''):','XY Kernel size:',sprintf('Z Kernel size for 3D; Y-dim for 2D\nleave empty for automatic calculation based on voxel size:'),'Sigma'};
answer = inputdlg(prompt,title,[1 30],def,'on');    
if size(answer) == 0; return; end; 
if isnan(str2double(answer{3}))
    kernel = str2double(answer{2});
else
    kernel = [str2double(answer{2}) str2double(answer{3})];
end

if strcmp(answer{1},'2d')
    options.fitType = 'Gaussian';
else
    options.fitType = 'Gaussian 3D';
end
options.hSize = kernel;
if isempty(answer{4})
    options.sigma = 1;
else
    options.sigma = str2double(answer{4});    
end

options.pixSize = handles.Img{handles.Id}.I.pixSize;
options.orientation = handles.Img{handles.Id}.I.orientation;
type = lower(type);
switch type
    case 'mask'
        wb = waitbar(0,'Please wait...','Name','Smoothing the mask','WindowStyle','modal');
        ib_do_backup(handles, 'mask', 1);
        options.dataType = '3D';
        mask = ib_getDataset('mask', handles, 4);
        for roi=1:numel(mask)
            mask{roi} = ib_doImageFiltering(mask{roi}, options);
        end
        ib_setDataset('mask', mask, handles, 4);
    case 'selection'
        wb = waitbar(0,'Smoothing the selection, please wait...','WindowStyle','modal');
        ib_do_backup(handles, 'selection', 1);
        options.dataType = '3D';
        selection = ib_getDataset('selection', handles, 4);
        for roi=1:numel(selection)
            selection{roi} = ib_doImageFiltering(selection{roi}, options);
        end
        ib_setDataset('selection', selection, handles, 4);
    case 'model'
        options.dataType = '3D';
        sel_model = get(handles.segmList,'Value');
        model_list = get(handles.segmList,'String');
        ib_do_backup(handles, 'model', 1);
        %if sel_model < 0
        %    wb = waitbar(0,'Smoothing all models, please wait...','WindowStyle','modal');
        %    start_no=1;
        %    end_no=numel(model_list);
        %else
            wb = waitbar(0,['Smoothing material: ' model_list{sel_model} ', please wait...'],'WindowStyle','modal');
            start_no=sel_model;
            end_no=sel_model;
        %end
        for object = start_no:end_no
            model = ib_getDataset('model', handles, 4, object);
            for roi=1:numel(model)
                model{roi} = ib_doImageFiltering(model{roi}, options);
            end
            ib_setDataset('model', model, handles, 4, object);
        end
end
delete(wb);
end