function ib_exportToFiji(handles)
% function ib_exportToFiji(handles)
% Export currently open dataset to Fiji
%
% @note requires Fiji to be installed (http://fiji.sc/Fiji).
%
% Parameters:
% handles: handles structure of im_browser
%
% Return values:

% Copyright (C) 16.05.2013 Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% part of Microscopy Image Browser, http:\\mib.helsinki.fi 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 


% check for MIJ
if exist('MIJ','class') == 8
    if ~isempty(ij.gui.Toolbar.getInstance)
        ij_instance = char(ij.gui.Toolbar.getInstance.toString);
        % -> ij.gui.Toolbar[canvas1,3,41,548x27,invalid]
        if numel(strfind(ij_instance, 'invalid')) > 0    % instance already exist, but not shown
            Miji_wrapper(true);     % wrapper to Miji.m file
        end
    else
        Miji_wrapper(true);     % wrapper to Miji.m file
    end
else
   Miji_wrapper(true);     % wrapper to Miji.m file
end

filename = handles.Img{handles.Id}.I.img_info('Filename');
[~, fn] = fileparts(filename);

%options.WindowStyle='modal';
%answer = inputdlg(sprintf('Please name for the dataset:'),'Set name',1,cellstr(fn),options);
answer = mib_inputdlg(NaN,'Please name for the dataset:','Set name',fn);
if isempty(answer); return; end;

pause(0.1);     % for some strange reason have to put pause here, otherwise everything is freezing...

% define type of the dataset
datasetTypeValue = get(handles.fijiconnectTypePop, 'value');
datasetTypeList = get(handles.fijiconnectTypePop, 'string');
datasetType = datasetTypeList{datasetTypeValue};

options.blockModeSwitch = 0;
img = handles.Img{handles.Id}.I.getData4D(datasetType, NaN, 4, NaN, options);

if size(img,3) == 1; img = squeeze(img); end;

if ndims(img) == 4
    img = permute(img, [1 2 4 3]);
    MIJ.createColor(answer{1}, img, 1);
elseif strcmp(handles.Img{handles.Id}.I.img_info('ColorType'),'truecolor')
    MIJ.createColor(answer{1}, img, 1);
else
    imp = MIJ.createImage(answer{1}, img, 0);
    imp.show
end

end

