function handles = ib_renderModelImaris(handles)
% function handles = ib_renderModelImaris(handles)
% Render a model in Imaris. 
%
% Parameters:
% handles: handles structure from im_browser
%
% Return values:
% handles:  handles structure from im_browser

% @note 
% uses IceImarisConnector bindings
% @b Requires:
% 1. set system environment variable IMARISPATH to the installation
% directory, for example "c:\tools\science\imaris"
% 2. restart Matlab
% 

%|
% @b Examples:
% @code handles = ib_setImarisDataset(handles.Img{handles.Id}.I.img, handles.Img{handles.Id}.I.img_info, handles);     // get dataset from imaris @endcode

% Copyright (C) 05.11.2014 Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% part of Microscopy Image Browser, http:\\mib.helsinki.fi 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% Written with a help of an old code SurfacesFromSegmentationImage.m by
% Igor Beati, Bitplane.
%
% Updates
% 



answer = inputdlg(sprintf('!!! ATTENTION !!!\n\nA volume that is currently open in Imaris will be removed!\nYou can preserve it by importing it into MIB and exporting it back to Imaris after the surface is generated.\n\nTo proceed further please define a smoothing factor,\na number, 0 or higher (IN IMAGE UNITS):'),'Smoothing factor',1,{'0'});
%answer = mib_inputdlg(NaN,sprintf('!!! ATTENTION !!!A volume that is currently open in Imaris will be removed!\nYou can preserve it by importing it into MIB and exporting it back to Imaris after the surface is generated.\n\nTo proceed further please define a smoothing factor,\na number, 0 or higher (IN IMAGE UNITS):'),'Smoothing factor','0');
if isempty(answer); return; end;
vSmoothing = str2double(answer{1});

% establishing connection to Imaris
wb = waitbar(0, 'Please wait...', 'Name', 'Connecting to Imaris');
if ~isfield(handles, 'connImaris')
    try
        handles.connImaris = IceImarisConnector(0);
    catch exception
        errordlg(sprintf('Could not connect to Imaris Server;\nPlease start Imaris and try again!'),'Missing Imaris');
        delete(wb);
        return;
    end
    if handles.connImaris.isAlive == 0
        % start Imaris
        handles.connImaris.startImaris();
    end
else
    if handles.connImaris.isAlive == 0
        % start Imaris
        handles.connImaris.startImaris();
    end
end
delete(wb);

wb = waitbar(0, 'Please wait...','Name','Rendering model in Imaris'); 
% define index of material to model, NaN - model all
if get(handles.seeAllMaterialsCheck, 'value') == 1
    materialStart = 1;  
    materialEnd = numel(handles.Img{handles.Id}.I.modelMaterialNames);  
    vNumberOfObjects = numel(handles.Img{handles.Id}.I.modelMaterialNames);
else
    materialStart = get(handles.segmList, 'value');  
    materialEnd = get(handles.segmList, 'value');  
    vNumberOfObjects = 1;
end
options.blockModeSwitch = 0;    % get full dataset
%model = handles.Img{handles.Id}.I.getData4D('model', NaN, 4, materialToModel, options);

for vIndex = materialStart:materialEnd
    waitbar(0.05,wb,sprintf('Generating %s\nPlease wait...',handles.Img{handles.Id}.I.modelMaterialNames{vIndex}));
    vBinaryImage = handles.Img{handles.Id}.I.getData4D('model', NaN, 4, vIndex, options);   % get material
    
    if isempty(handles.connImaris.mImarisApplication.GetDataSet)
        % allocate space for a new dataset
        aDataSet = handles.connImaris.createDataset(class(vBinaryImage), size(vBinaryImage,2), size(vBinaryImage,1), size(vBinaryImage,3), 1, 1);
        aDataSet.SetTimePoint(0, '2006-05-11 13:30:00.000');
    else
        % get handle to the existing dataset
        %aDataSet = handles.connImaris.mImarisApplication.GetDataSet();
        %timePoint = aDataSet.GetTimePoint(0);
        aDataSet = handles.connImaris.createDataset(class(vBinaryImage), size(vBinaryImage,2), size(vBinaryImage,1), size(vBinaryImage,3), 1, 1);
        %aDataSet.SetTimePoint(0, timePoint);
        aDataSet.SetTimePoint(0, '2006-05-11 13:30:00.000');
    end
    
    aSurfaces = handles.connImaris.mImarisApplication.GetFactory.CreateSurfaces; % create resulting surface
    
    % send dataset to Imaris
    
    sizeX = size(vBinaryImage,2);
    sizeY = size(vBinaryImage,1);
    sizeZ = size(vBinaryImage,3);
    
    blockSizeX = 512;
    blockSizeY = 512;
    blockSizeZ = 512;
    
    useBlockMode = 0;
    if sizeX*sizeY*sizeZ > 134217728
        useBlockMode = 1;
    end
    
    if useBlockMode == 0
        handles.connImaris.setDataVolumeRM(vBinaryImage, 0, 0);
    else
        for z=0:ceil(sizeZ/blockSizeZ)-1
            for y=0:ceil(sizeY/blockSizeY)-1
                for x=0:ceil(sizeX/blockSizeX)-1
                    imgBlock = vBinaryImage(...
                        1+blockSizeY*y:min(blockSizeY+blockSizeY*y, sizeY) ,...
                        1+blockSizeX*x:min(blockSizeX+blockSizeX*x, sizeX) ,...
                        1+blockSizeZ*z:min(blockSizeZ+blockSizeZ*z, sizeZ));
                    
                    handles.connImaris.mib_setDataSubVolumeRM(imgBlock,...
                        blockSizeX*x, blockSizeY*y, blockSizeZ*z,...
                        0, 0,...
                        size(imgBlock,2), size(imgBlock,1), size(imgBlock,3));
                end
            end
        end
    end
    
    % update the bounding box
    bb = handles.Img{handles.Id}.I.getBoundingBox();    % bb[xMin, xMax, yMin, yMax, zMin, zMax]
    handles.connImaris.mImarisApplication.GetDataSet.SetExtendMinX(bb(1));
    handles.connImaris.mImarisApplication.GetDataSet.SetExtendMaxX(bb(2));
    handles.connImaris.mImarisApplication.GetDataSet.SetExtendMinY(bb(3));
    handles.connImaris.mImarisApplication.GetDataSet.SetExtendMaxY(bb(4));
    handles.connImaris.mImarisApplication.GetDataSet.SetExtendMinZ(bb(5));
    % fix of a problem of different calculations of the bounding box for a
    % single slice and Z-stack
    if handles.Img{handles.Id}.I.no_stacks > 1
        handles.connImaris.mImarisApplication.GetDataSet.SetExtendMaxZ(bb(6)+handles.Img{handles.Id}.I.pixSize.z);
    else
        handles.connImaris.mImarisApplication.GetDataSet.SetExtendMaxZ(bb(6));
    end
    
    % generate surface
    vSurfaces = handles.connImaris.mImarisApplication.GetImageProcessing.DetectSurfaces(...
        aDataSet, [], 0, vSmoothing, 0, 0, .5, '');     
    if vSurfaces.GetNumberOfSurfaces == 0
        button = questdlg(sprintf('!!! Warning !!!\nThe smoothing factor is too large!\nContinue with the smoothing factor == 0?'),'Too large smoothing','Continue','Cancel','Cancel');
        if strcmp(button, 'Cancel')
            delete(wb);
            return;
        else
            vSmoothing = 0;
            vSurfaces = handles.connImaris.mImarisApplication.GetImageProcessing.DetectSurfaces(...
                aDataSet, [], 0, vSmoothing, 0, 0, .5, '');     
        end
    end
    
    for vSurface = 1:vSurfaces.GetNumberOfSurfaces
        % copy this surface to the main surface container
        %vTimeIndex = vSurfaces.GetTimeIndex(vSurface - 1);
        vTimeIndex = 0;
        vVertices = vSurfaces.GetVertices(vSurface - 1);
        vTriangles = vSurfaces.GetTriangles(vSurface - 1);
        vNormals = vSurfaces.GetNormals(vSurface - 1);
        if ~isempty(vVertices)
            aSurfaces.AddSurface(vVertices, vTriangles, vNormals, vTimeIndex);
        end
    end
    
    % set name for the surface
    aSurfaces.SetName(handles.Img{handles.Id}.I.modelMaterialNames{vIndex});
    % set color for the surface
    ColorRGBA = [handles.Img{handles.Id}.I.modelMaterialColors(vIndex,:), 0];
    ColorRGBA = handles.connImaris.mapRgbaVectorToScalar(ColorRGBA);
    aSurfaces.SetColorRGBA(ColorRGBA);
    
    %aSurfaces.SetTimePoint(0, '2006-05-11 13:30:00.000');
    timePoint = aDataSet.GetTimePoint(0);
    aSurfaces.SetTimePoint(0, timePoint);
    
    % add surface to scene
    handles.connImaris.mImarisApplication.GetSurpassScene.AddChild(aSurfaces, -1);
    waitbar(vIndex / vNumberOfObjects, wb);
end
delete(wb);
end