function varargout = selectHDFSeries(varargin)
% function varargout = selectHDFSeries(varargin)
% selectHDFSeries function is responsible for selection of series in HDF files
%
% selectHDFSeries contains MATLAB code for selectHDFSeries.fig 

% Copyright (C) 21.11.2013 Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% part of Microscopy Image Browser, http:\\mib.helsinki.fi 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 

% Last Modified by GUIDE v2.5 17-Jan-2012 16:41:42

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @selectHDFSeries_OpeningFcn, ...
                   'gui_OutputFcn',  @selectHDFSeries_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT
end

% --- Executes just before selectHDFSeries is made visible.
function selectHDFSeries_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to selectHDFSeries (see VARARGIN)

% Choose default command line output for selectHDFSeries
handles.output = 'Cancel';  
handles.output2 = 1;  % meta data switch
handles.output3 = [NaN NaN NaN NaN NaN]; % dimensions of the selected dataset xyczt

if iscell(varargin{1})
    handles.filename = cell2mat(varargin{1});    % file name to get image dataset
else
        handles.filename = varargin{1};
end

info = h5info(handles.filename);
index=1;

if ~isempty(info.Datasets)
    for dataset = 1:numel(info.Datasets)
        datasetName = info.Datasets(dataset).Name;
        if datasetName(1) ~= '/'; datasetName = ['/' datasetName]; end;
        data(index,1) = {datasetName};
        datadim = info.Datasets(dataset).Dataspace.Size;
        for dim=1:numel(datadim)
            data(index,1+dim) = {num2str(datadim(dim))};
        end
    data(index,7) = {info.Datasets(dataset).Datatype.Class};
    index = index + 1;    
    end
end
for group=1:numel(info.Groups)
    for dataset=1:numel(info.Groups(group).Datasets)
        data(index,1) = {[info.Groups(group).Name '/' info.Groups(group).Datasets(dataset).Name]};
        datadim = info.Groups(group).Datasets(dataset).Dataspace.Size;
        for dim=1:numel(datadim)
            data(index,1+dim) = {num2str(datadim(dim))};
        end
        data(index,7) = {info.Groups(group).Datasets(dataset).Datatype.Class};
        index = index + 1;
    end
end
set(handles.seriesTable,'Data',data);

% Update handles structure
guidata(hObject, handles);

% Determine the position of the dialog - centered on the callback figure
% if available, else, centered on the screen
FigPos=get(0,'DefaultFigurePosition');
OldUnits = get(hObject, 'Units');
set(hObject, 'Units', 'pixels');
OldPos = get(hObject,'Position');
FigWidth = OldPos(3);
FigHeight = OldPos(4);
if isempty(gcbf)
    ScreenUnits=get(0,'Units');
    set(0,'Units','pixels');
    ScreenSize=get(0,'ScreenSize');
    set(0,'Units',ScreenUnits);

    FigPos(1)=1/2*(ScreenSize(3)-FigWidth);
    FigPos(2)=2/3*(ScreenSize(4)-FigHeight);
else
    GCBFOldUnits = get(gcbf,'Units');
    set(gcbf,'Units','pixels');
    GCBFPos = get(gcbf,'Position');
    set(gcbf,'Units',GCBFOldUnits);
    FigPos(1:2) = [(GCBFPos(1) + GCBFPos(3) / 2) - FigWidth / 2, ...
                   (GCBFPos(2) + GCBFPos(4) / 2) - FigHeight / 2];
end
FigPos(3:4)=[FigWidth FigHeight];
set(hObject, 'Position', FigPos);
set(hObject, 'Units', OldUnits);

% Make the GUI modal
set(handles.selectHDFSeries,'WindowStyle','modal')

% UIWAIT makes selectHDFSeries wait for user response (see UIRESUME)
uiwait(handles.selectHDFSeries);
end

% --- Outputs from this function are returned to the command line.
function varargout = selectHDFSeries_OutputFcn(hObject, eventdata, handles)
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;
varargout{2} = handles.output2;
for i = 1:numel(handles.output3)
    if isnan(handles.output3(i))
        handles.output3(i) = 0;
    end
end
varargout{3} = handles.output3;

% The figure can be deleted now
delete(handles.selectHDFSeries);
end

% --- Executes on button press in continueBtn.
function continueBtn_Callback(hObject, eventdata, handles)
% hObject    handle to continueBtn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Update handles structure
guidata(hObject, handles);

% Use UIRESUME instead of delete because the OutputFcn needs
% to get the updated handles structure.
uiresume(handles.selectHDFSeries);
end


% --- Executes on button press in cancelBtn.
function cancelBtn_Callback(hObject, eventdata, handles)
% hObject    handle to cancelBtn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

handles.output = 'Cancel';

% Update handles structure
guidata(hObject, handles);

% Use UIRESUME instead of delete because the OutputFcn needs
% to get the updated handles structure.
uiresume(handles.selectHDFSeries);
end

% --- Executes when user attempts to close selectHDFSeries.
function selectHDFSeries_CloseRequestFcn(hObject, eventdata, handles)
% hObject    handle to selectHDFSeries (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if isequal(get(hObject, 'waitstatus'), 'waiting')
    % The GUI is still in UIWAIT, us UIRESUME
    uiresume(hObject);
else
    % The GUI is no longer waiting, just close it
    delete(hObject);
end
end


% --- Executes when selected cell(s) is changed in seriesTable.
function seriesTable_CellSelectionCallback(hObject, eventdata, handles)
% hObject    handle to seriesTable (see GCBO)
% eventdata  structure with the following fields (see UITABLE)
%	Indices: row and column indices of the cell(s) currently selecteds
% handles    structure with handles and user data (see GUIDATA)
tableData = get(handles.seriesTable,'Data');
rowData = tableData(eventdata.Indices(1),:);
set(handles.selectedSeriesText,'String', rowData(1));
handles.output = rowData(1);

dim_xyczt = zeros(1,numel(rowData)-2);
for i=1:numel(rowData)-2
    dim_xyczt(i) = str2double(rowData{i+1});
end

handles.output3 = dim_xyczt;
% Update handles structure
guidata(hObject, handles);
end


% --- Executes on button press in parametersCheckbox.
function parametersCheckbox_Callback(hObject, eventdata, handles)
% hObject    handle to parametersCheckbox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

handles.output2 = get(hObject, 'Value');
% Update handles structure
guidata(hObject, handles);
end
