function handles = ib_invertImage(handles, col_channel)
% function handles = ib_invertImage(handles, col_channel)
% Invert image
%
% Parameters:
% handles: structure with handles of im_browser.m
% col_channel: [@em optional] a number of color channel to invert
%
% Return values:
% handles: structure with handles of im_browser.m

% Copyright (C) 03.03.2014 Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% part of Microscopy Image Browser, http:\\mib.helsinki.fi 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 

if nargin < 2
    if numel(get(handles.ColChannelCombo,'string')) > 2
        if numel(handles.Img{handles.Id}.I.slicesColor) == 1
            col_channel = handles.Img{handles.Id}.I.slicesColor;
        else
            col_channel = get(handles.ColChannelCombo,'value') - 1;
        end
        strText = sprintf('Would you like to invert only channel: %d or all channels?', col_channel);
        button = questdlg(strText,'Invert Image','All channels',sprintf('Channel %d', col_channel),'Cancel','All channels');
        if strcmp(button, 'Cancel'); return; end;
        if strcmp(button, 'All channels')
            col_channel = 0;
        else
            col_channel = get(handles.ColChannelCombo,'value')-1;
        end
    else
        col_channel = 0;
    end
end

if col_channel == 0
    color_start = 1;
    color_end = handles.Img{handles.Id}.I.colors;
else
    color_start = col_channel;
    color_end = col_channel;
end
maxval = intmax(class(handles.Img{handles.Id}.I.img));
ib_do_backup(handles, 'image', 1);
img = ib_getDataset('image', handles, 4, col_channel);
for roi=1:numel(img)
    if color_start==color_end
        img{roi}(:,:,1,:) = maxval - img{roi}(:,:,1,:);
    else
        for color=color_start:color_end
            img{roi}(:,:,color,:) = maxval - img{roi}(:,:,color,:);
        end
    end
end
ib_setDataset('image', img, handles, 4, col_channel);
log_text = sprintf('Invert, ColCh: %d-%d', color_start, color_end);
handles.Img{handles.Id}.I.updateImgInfo(log_text);
end