%% Image Menu
% Image processing functions
%
%
% *Back to* <im_browser_product_page.html *Index*> |*-->*| <im_browser_user_guide.html *User Guide*> |*-->*| <ug_gui_menu.html *Menu*>
%% Mode
% Allows to change mode of the shown dataset, the following options are available:
% 
% * *Grayscale*, converts image to grayscale by removing any color
% information.
% * *RGB Color*, converts image to the RGB color space.
% * *HSV Color*, converts image to the HSV (hue, saturation, value) color space.
% * *Indexed*, converts image to the indexed colors (|not implemented for True
% Color images|).
% * *8 bit*, convert dataset to the 8 bit format, the image intensities are
% scaled to preserve contrast of the original dataset.
% * *16 bit*, convert dataset to the 16 bit format, the image intensities are
% scaled to preserve contrast of the original dataset.
% * *32 bit*, convert dataset to the 32 bit format, the image intensities are
% scaled to preserve contrast of the original dataset.
% 
%% Color Channels
% Perform some actions with color channels of the image
% 
% * *Swap channels...*, allows to swap two color channels between each other
% * *Delete channel...*, deletes specified color channel from the dataset.
%
% It is also possible to delete color channel by pressing the |Ctrl| key and clicking on the desired color
% channel in the |Colors| table in the <ug_panel_view_settings.html View settings panel>.
%
%% Contrast
% Adjust contrast of the dataset
%%
% 
% * *Linear contrast*, linear contrast adjustment for current or all
% slices. It is however recommended to use the |Display| button in the <ug_panel_view_settings.html View settings panel>.
% * *Contrast-limited adaptive histogram equalization for current image
% only*, CLAHE contrast equalization for the current slice. For
% details see documentation for Matlab function |adapthisteq|. 
% * *Contrast-limited adaptive histogram equalization for all images*, CLAHE contrast equalization for the whole dataset. For
% details see documentation for Matlab function |adapthisteq|. 
% * *Normalize layers*, normalization of image intensities between the
% slices.  A) calculates mean intensity and standard deviation (std) for the whole dataset;
% B) calculates mean intensities and standard deviation for each image; C) shifts each image 
% based on difference between mean values of that image and the whole dataset, 
% plus stretches it based on ratio between standard deviation of the whole dataset and each image
% * *Normalize layers based on masked areas*, essentially similar to the _Normalize layers_ mode, except that all values are
% calculated from the masked areas only.
% * *Normalize based on masked background*, normalizes image intensities between the
% slices using background areas that should be masked. A) calculates mean 
% intensity for the masked area for the whole dataset; B) calculates mean 
% intensities for the masked area for each image; C) shifts each image based 
% on difference between mean values of the image and the whole dataset
% 
%% Invert image
% Invert image intensities, shortcut |Ctrl+i|.
%% Morphological operations
% This section contains number of morphological operations that can be
% applied to images. The processed image may be also added or subtracted
% from the existing image (see the settings in the |Additional action to
% the result| panel).
% 
% <<images\menuImageMorphOps.jpg>>
% 
% List of available morphological operations
%
% * *Bottom-hat filtering (imbothat)* computes the morphological closing of the image (using imclose`) and then subtracts the result from the original image 
% * *Clear border (imclearborder)* suppresses light structures connected to image border
% * *Morphological closing (imclose)* morphologically closes the image: a dilation followed by an erosion
% * *Dilate image (imdilate)* 
% * *Erode image (imerode)* 
% * *Fill regions (imfill)* fills holes in the image, where a hole is defined as an area of dark pixels surrounded by lighter pixels
% * *H-maxima transform (imhmax)* suppresses all maxima in the image whose height is less than H
% * *H-mminima transform (imhmin)* uppresses all minima in the image whose depth is less than H
% * *Morphological opening (imopen)* morphologically opens image: an erosion followed by a dilation
% * *Top-hat filtering (imtophat)* computes the morphological opening of the image (using imopen) and then subtracts the result from the original image
%
%% Intensity profile
% Generate an intensity profile of the image data. The profiles can be
% obtained in two modes:
%
% * *Line* 
% * *Arbitrary* 
%
%% Histogram
% Show histogram of image intensities, see details in |imcontrast| function
% of Matlab.
%
%
%
% *Back to* <im_browser_product_page.html *Index*> |*-->*| <im_browser_user_guide.html *User Guide*> |*-->*| <ug_gui_menu.html *Menu*>


